#include "api/df_timeboy_lists.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE void DF_API_CALL DFCClearTimeboyListBuffer( int connectionNumber, int groupNumber, int number )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    channelBridge->logBegin( __func__, "\t%i\t%i", groupNumber, number );

    // zulaessige Bereiche pruefen
    if ( groupNumber < -1 || groupNumber > 19 || number < -1 || number > 19 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "" );
        return;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->ClearTimeboyListBuffer( groupNumber, number );

    channelBridge->logEnd( "" );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeTimeboyList( int connectionNumber, int groupNumber, int number, int recordCount, int size, const unsigned char *buffer, int flags, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i\t%i\t%i\t%i", groupNumber, number, recordCount, size );

    // zulaessige Bereiche pruefen
    if ( groupNumber < 0 || groupNumber > 19 || number < 0 || number > 19 || buffer == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t%i\t%i\t(%i, %i)", flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->MakeTimeboyList( groupNumber, number, recordCount, buffer, size, flags, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( 0, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( 0 ) );
    }

    channelBridge->logEnd( "\t\t%i\t%i\t(%i, %i)", flags, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadTimeboyList( int connectionNumber, int deviceAddress, int groupNumber, int number, int unused_setTo0, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%i\t%i\t%i", groupNumber, number, unused_setTo0 );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0
         || groupNumber < -1 || groupNumber > 19 || number < -1 || number > 19 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Timeboy Listenuebertragung wird erst ab 04.01.xx unterstuetzt.
    unsigned char protocolVersion = channelBridge->getProtocolVersion( deviceAddress );
    if ( protocolVersion == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_UNREACHABLE, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }
    else if ( protocolVersion < 4 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_VERSION_FIRMWARE, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int iRes;
    std::string version;
    // Firmwareversion lesen
    if ( (iRes = channelBridge->getVersionFirmware( deviceAddress, version )) != IV_ERROR_SUCCESS )
    {
        channelBridge->setLastErrorNumber( deviceAddress, (iRes < 0) ? -iRes : IV_ERROR_VERSION_FIRMWARE, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Listendaten wegschreiben.
    if ( channelBridge->logIsWriteList() )
    {
        char fileNameBin[_MAX_PATH];
        char basePath[340];
        int listID, listNum;
        time_t t;
        tm *ltm;

        t = time( nullptr );
        ltm = localtime( &t );

        for ( listID = 0; listID < MAX_RECORD_DESCRIPTION; ++listID )
        {
            for ( listNum = 0; listNum < MAX_RECORD_DESCRIPTION; ++listNum )
            {
                // Dateiname der Logdatei
                std::string logBasePath = channelBridge->logGetBasePath( basePath );
                int res = snprintf( fileNameBin, sizeof( fileNameBin ) - 1, LOG_LIST_FILENAME_BIN, logBasePath.c_str(),
                                    ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                                    ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionNumber, (char)(0x30 + listID), listNum );

                if ( res > static_cast<int>(sizeof( fileNameBin )) )
                {
                    assert( false );
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_OPEN, errorNumber );
                    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
                    return 0;
                }

                // Daten in die Datei schreiben.
                channelBridge->WriteListTimeboyToBinFile( listID, listNum, fileNameBin );
            }
        }
    }

    int l_error_number = IV_ERROR_SUCCESS;
    iRes = channelBridge->writeTimeboyListData( deviceAddress, groupNumber, number, unused_setTo0, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}
